import '../beevoter_comm.dart';
import 'controller.dart';
import 'model.dart';
import 'dart:convert';

class BeeVoterQRCode {
	String url;
	String userContact;
	CredentialType userContactType;

	BeeVoterQRCode(String scan) {
		Map<String, dynamic> json = jsonDecode(scan);
		url = json['url'];
		userContact = json['userContact'];
		var userContactType = json['userContactType'];
		switch(userContactType) {
			case 0:
				this.userContactType = CredentialType.email();
				break;
			case 1:
				this.userContactType = CredentialType.email();
				break;
			case 2:
				this.userContactType = CredentialType.document();
				break;
		}
	}
}

abstract class PoolStationAction extends ConnectionAction {
	ConnectionInfo _ballotBoxConnectionInfo;
	BeeVoterController beeVoterController;

	bool connected() {
		return beeVoterController != null;
	}
	PoolStationAction(ConnectionInfo connectionInfo) {
		this._ballotBoxConnectionInfo = connectionInfo;
		_ballotBoxConnectionInfo.createConnection(this);
	}
	Future connectToBeevoter(BeeVoterQRCode qrcode, String password) async {
		this.beeVoterController = new BeeVoterController(qrcode.url);
		await this.beeVoterController.startConnection();
		await this.beeVoterController.login(qrcode.userContact, qrcode.userContactType , password);
	}

	Future onReceive(String localIp, int commandType, String command) async {
		switch(commandType) {
			case FaceToFaceCommand.requestBallotBoxConfiguration:
				// var cmd = RequestBallotBoxConfiguration.create(string2JsModel(command));
				initializeBallotBox();
				break;
			case FaceToFaceCommand.setBallotBoxStatus:
				setBallotBoxStatus(SetBallotBoxStatus.create(string2JsModel(command)));
				break;
		}
	}
	Future onConnectionClosed(String localIp) async {
		// finishBallotBox();
	}
	void initializeBallotBox() {
		if (beeVoterController == null || beeVoterController.initializeResponse == null)
			throw new Exception('Not initialized');
		_ballotBoxConnectionInfo.send(FaceToFaceCommand.initializeBallotBox, jsModel2String(beeVoterController.initializeResponse.toJson()));
	}
	void getBallotBoxStatus() {
		if (beeVoterController == null || beeVoterController.initializeResponse == null)
			throw new Exception('Not initialized');
		_ballotBoxConnectionInfo.send(FaceToFaceCommand.getBallotBoxStatus, jsModel2String(new GetBallotBoxStatus().toJson()));
	}
	void sendVoterToBallotBox(VoterModel voter) {
		if (beeVoterController == null || beeVoterController.initializeResponse == null)
			throw new Exception('Not initialized');
		_ballotBoxConnectionInfo.send(FaceToFaceCommand.sendVoterToBallotBox, jsModel2String(new VoterToBallotBox(voter).toJson()));
	}
	void cancelVote() {
		if (beeVoterController == null || beeVoterController.initializeResponse == null)
			throw new Exception('Not initialized');
		_ballotBoxConnectionInfo.send(FaceToFaceCommand.cancelVote, jsModel2String(new CancelVote().toJson()));
	}
	void finishBallotBox() {
		if (beeVoterController == null || beeVoterController.initializeResponse == null)
			throw new Exception('Not initialized');
		_ballotBoxConnectionInfo.send(FaceToFaceCommand.finishBallotBox, jsModel2String(new FinishBallotBox().toJson()));
	}
	void setBallotBoxStatus(SetBallotBoxStatus cmd);
}
abstract class BallotBoxAction extends ConnectionAction {
	ConnectionInfo _poolingStationConnectionInfo;
	InitializeResponse initializeResponse;
	int lastStatus = 0;

	BallotBoxAction(ConnectionInfo connectionInfo) {
		this._poolingStationConnectionInfo = connectionInfo;
		_poolingStationConnectionInfo.createConnection(this);
	}

	Future onReceive(String localIp, int commandType, String command) async {
		switch(commandType) {
			case FaceToFaceCommand.initializeBallotBox:
				this.initializeResponse = InitializeResponse.create(string2JsModel(command));
				break;
			case FaceToFaceCommand.getBallotBoxStatus:
				// var cmd = GetBallotBoxStatus.create(string2JsModel(command));
				setBallotBoxStatus(lastStatus);
				break;
			case FaceToFaceCommand.cancelVote:
				onCancelVote(CancelVote.create(string2JsModel(command)));
				break;
			case FaceToFaceCommand.finishBallotBox:
				onFinishBallotBox(FinishBallotBox.create(string2JsModel(command)));
				break;
		}
	}
	Future onConnectionClosed(String localIp) async {
	}
	void castVote(CastVote vote) {
		if (initializeResponse == null)
			throw new Exception('Not initialized');
		_poolingStationConnectionInfo.send(FaceToFaceCommand.castVote, jsModel2String(vote.toJson()));
	}
	void setBallotBoxStatus(int status) {
		this.lastStatus = status;
		if (initializeResponse == null)
			throw new Exception('Not initialized');
		_poolingStationConnectionInfo.send(FaceToFaceCommand.setBallotBoxStatus, jsModel2String(new SetBallotBoxStatus(status).toJson()));
	}
	void onCancelVote(CancelVote cmd);
	void onFinishBallotBox(FinishBallotBox cmd);
}

abstract class FaceToFaceCommand {
	static const int requestBallotBoxConfiguration=1;
	static const int initializeBallotBox=2;
	static const int getBallotBoxStatus=3;
	static const int setBallotBoxStatus=4;
	static const int sendVoterToBallotBox=5;
	static const int castVote=6;
	static const int cancelVote=7;
	static const int finishBallotBox=8;

	String signature;
	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = new Map<String, dynamic>();
		json['id'] = this.signature;
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		this.signature = json['id'];
	}

	static FaceToFaceCommand create(FaceToFaceCommand command, Map<String, dynamic> json) {
		if (json != null) command.fromJson(json);
		return command;
	}
}
class RequestBallotBoxConfiguration extends FaceToFaceCommand {
	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
	}

	static RequestBallotBoxConfiguration create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new RequestBallotBoxConfiguration(), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class InitializeBallotBox extends FaceToFaceCommand {
	InitializeResponse initializeResponse;
	String poolWorkerPassword;

	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		json['initializeResponse'] = this.initializeResponse.toJson();
		json['poolWorkerPassword'] = this.poolWorkerPassword;
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
		this.initializeResponse = InitializeResponse.create(json['initializeResponse']);
		this.poolWorkerPassword = json['poolWorkerPassword'];
	}

	static InitializeBallotBox create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new InitializeBallotBox(), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class GetBallotBoxStatus extends FaceToFaceCommand {
	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
	}

	static GetBallotBoxStatus create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new GetBallotBoxStatus(), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class SetBallotBoxStatus extends FaceToFaceCommand {
	int status;
	SetBallotBoxStatus(this.status);

	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		json['status'] = this.status;
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
		this.status = json['status'];
	}

	static SetBallotBoxStatus create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new SetBallotBoxStatus(0), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class VoterToBallotBox extends FaceToFaceCommand {
	VoterModel voter;
	VoterToBallotBox(this.voter);

	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		json['voter'] = this.voter.toJson();
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
		this.voter = VoterModel.create(json['voter']);
	}

	static VoterToBallotBox create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new VoterToBallotBox(null), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class CastVote extends FaceToFaceCommand {
	CastVotesRequestModel vote;

	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		json['vote'] = this.vote.toJson();
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
		this.vote = CastVotesRequestModel.create(json['vote']);
	}

	static CastVote create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new CastVote(), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class CancelVote extends FaceToFaceCommand {
	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
	}

	static CancelVote create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new CancelVote(), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
class FinishBallotBox extends FaceToFaceCommand {
	Map<String, dynamic> toJson() {
		final Map<String, dynamic> json = super.toJson();
		return json;
	}

	fromJson(Map<String, dynamic> json) {
		super.fromJson(json);
	}

	static FinishBallotBox create(Map<String, dynamic> json) {
		var ret = FaceToFaceCommand.create(new FinishBallotBox(), json);
		if (json != null) ret.fromJson(json);
		return ret;
	}
}
